﻿#pragma once

#include <memory>
#include <string>
#include <format>

#include "Core.h"

namespace spdlog
{
    class logger;
}

struct LoggerImpl;

class FLUX_API Logger
{
public:
    Logger();
    ~Logger();

    void LogTrace(const std::string& message);
    void LogInfo(const std::string& message);
    void LogWarn(const std::string& message);
    void LogError(const std::string& message);
    void LogFatal(const std::string& message);

    template<typename... Args>
    void LogTrace(std::format_string<Args...> fmt, Args&&... args)
    {
        LogTrace(std::format(fmt, std::forward<Args>(args)...));
    }

    template<typename... Args>
    void LogInfo(std::format_string<Args...> fmt, Args&&... args)
    {
        LogInfo(std::format(fmt, std::forward<Args>(args)...));
    }

    template<typename... Args>
    void LogWarn(std::format_string<Args...> fmt, Args&&... args)
    {
        LogWarn(std::format(fmt, std::forward<Args>(args)...));
    }

    template<typename... Args>
    void LogError(std::format_string<Args...> fmt, Args&&... args)
    {
        LogError(std::format(fmt, std::forward<Args>(args)...));
    }

    template<typename... Args>
    void LogFatal(std::format_string<Args...> fmt, Args&&... args)
    {
        LogFatal(std::format(fmt, std::forward<Args>(args)...));
    }

private:
    spdlog::logger* GetLogger() const;

    LoggerImpl& _impl;
};

#define LOG_TRACE(...) ::FluxEngine::GetLogger()->LogTrace(__VA_ARGS__)
#define LOG_INFO(...)  ::FluxEngine::GetLogger()->LogInfo (__VA_ARGS__)
#define LOG_WARN(...)  ::FluxEngine::GetLogger()->LogWarn (__VA_ARGS__)
#define LOG_ERROR(...) ::FluxEngine::GetLogger()->LogError(__VA_ARGS__)
#define LOG_FATAL(...) ::FluxEngine::GetLogger()->LogFatal(__VA_ARGS__)
